import os

from playground.config import SIGNED_REQUESTS_DIR, SIGNING_KEY_DIR
from thenewboston.accounts.key_files import read_signing_key_file
from thenewboston.utils.files import write_json
from thenewboston.utils.format import format_address
from thenewboston.utils.network import post
from thenewboston.utils.signed_requests import generate_signed_request

"""
Primary validator updated notice from bank
POST /primary_validator_updated
"""


def run(send_to_cv=False):
    """
    Generate primary validator updated notice (from bank)
    """

    # Signed request
    sk = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'bank_nid'))

    new_pv_data = {
        'ip_address': '192.168.1.20',
        'port': '8000',
        'protocol': 'http'
    }

    signed_request = generate_signed_request(
        data=new_pv_data,
        nid_signing_key=sk
    )

    if send_to_cv:
        send_request_to_cv(signed_request)

    write_json(
        os.path.join(SIGNED_REQUESTS_DIR, 'primary-validator-updated-request.json'),
        signed_request
    )


def send_request_to_cv(signed_request):
    """
    Send request to CV
    """

    node_address = format_address(
        ip_address='192.168.1.74',
        port=8000,
        protocol='http'
    )
    url = f'{node_address}/primary_validator_updated'
    results = post(url=url, body=signed_request)

    if isinstance(results, dict):
        for k, v in results.items():
            print(f'{k}: {v}')

    print(results)

    write_json(
        os.path.join(SIGNED_REQUESTS_DIR, 'primary-validator-updated-response.json'),
        results
    )


if __name__ == '__main__':
    run(send_to_cv=True)



from django.core.validators import MinValueValidator
from django.db import models

from thenewboston.constants.network import ACCOUNT_FILE_HASH_LENGTH, BLOCK_IDENTIFIER_LENGTH, MIN_POINT_VALUE
from thenewboston.models.network_node import NetworkNode
from thenewboston.utils.validators import validate_is_real_number


class NetworkValidator(NetworkNode):
    root_account_file = models.URLField(max_length=1024)
    root_account_file_hash = models.CharField(max_length=ACCOUNT_FILE_HASH_LENGTH)
    seed_block_identifier = models.CharField(blank=True, max_length=BLOCK_IDENTIFIER_LENGTH)

    # Confirmation rate (used by confirmation validators only)
    daily_confirmation_rate = models.DecimalField(
        blank=True,
        decimal_places=16,
        max_digits=32,
        null=True,
        validators=[
            MinValueValidator(MIN_POINT_VALUE),
            validate_is_real_number
        ]
    )

    class Meta:
        abstract = True



from operator import itemgetter

from prettytable import PrettyTable


def display_item(item, *, excluded=None, title=None):
    """
    Display dictionary in table format
    """

    if not item:
        return

    pt = PrettyTable(header=False)

    excluded = excluded or []
    items = [[k, v] for k, v in item.items() if k not in excluded]

    for item in items:
        pt.add_row(item)

    if title:
        print(title)

    print(f'{pt}\n')


def display_list(list_items, *, ascending=True, excluded=None, order_by=None, title=None):
    """
    Display formatted table of list items
    """

    if not list_items:
        return

    pt = PrettyTable()
    list_headers = get_list_headers(list_items)
    list_headers = [i for i in list_headers if i not in excluded] if excluded else list_headers
    pt.field_names = list_headers

    if order_by:
        list_items = sorted(list_items, key=itemgetter(order_by), reverse=not ascending)

    for item in list_items:
        pt.add_row([item.get(i) for i in list_headers])

    if title:
        print(title)

    print(f'{pt}\n')


def get_list_headers(list_items):
    """
    Get headers from a list of dictionaries
    """

    first_item = list_items[0]
    return first_item.keys()



from rest_framework import serializers

from thenewboston.constants.network import VERIFY_KEY_LENGTH


class NetworkTransactionSerializer(serializers.Serializer):
    amount = serializers.DecimalField(max_digits=32, decimal_places=16)
    recipient = serializers.CharField(max_length=VERIFY_KEY_LENGTH)

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass

    @staticmethod
    def validate_amount(amount):
        """
        Check that amount is not 0
        """

        if amount == 0:
            raise serializers.ValidationError('Tx amount can not be 0 (Tx should be excluded)')

        return amount

    @staticmethod
    def validate_recipient(recipient):
        """
        Check recipient length
        """

        if len(recipient) != VERIFY_KEY_LENGTH:
            raise serializers.ValidationError(f'recipient must be {VERIFY_KEY_LENGTH} characters long')

        return recipient



from rest_framework import serializers

from thenewboston.constants.network import VERIFY_KEY_LENGTH


class AccountBalanceSerializer(serializers.Serializer):
    account_number = serializers.CharField(max_length=VERIFY_KEY_LENGTH)
    balance = serializers.DecimalField(max_digits=32, decimal_places=16)

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass



from uuid import uuid4

from django.core.validators import MinValueValidator
from django.db import models

from thenewboston.constants.network import MIN_POINT_VALUE, VERIFY_KEY_LENGTH
from thenewboston.utils.validators import validate_is_real_number


class NetworkTransaction(models.Model):
    id = models.UUIDField(default=uuid4, editable=False, primary_key=True)
    amount = models.DecimalField(
        decimal_places=16,
        default=MIN_POINT_VALUE,
        max_digits=32,
        validators=[
            MinValueValidator(MIN_POINT_VALUE),
            validate_is_real_number
        ]
    )
    recipient = models.CharField(max_length=VERIFY_KEY_LENGTH)

    class Meta:
        abstract = True



from rest_framework import serializers

from thenewboston.constants.network import BALANCE_LOCK_LENGTH
from thenewboston.serializers.network_transaction import NetworkTransactionSerializer


class MessageSerializer(serializers.Serializer):
    balance_key = serializers.CharField(max_length=BALANCE_LOCK_LENGTH)
    txs = NetworkTransactionSerializer(many=True)

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass



import unittest

import pycodestyle


class TestApplication(unittest.TestCase):

    def test_style(self):
        """
        Test PEP 8 style conventions
        E501 - Line too long (82 > 79 characters)
        W504 - Line break occurred after a binary operator
        """

        style = pycodestyle.StyleGuide(ignore=['E501', 'W504'])
        result = style.check_files([
            'tests/',
            'thenewboston/',
        ])
        self.assertEqual(result.total_errors, 0)



import os

from playground.config import PV_NID_ACCOUNT_NUMBER, SIGNED_REQUESTS_DIR, SIGNING_KEY_DIR
from thenewboston.accounts.key_files import read_signing_key_file
from thenewboston.utils.files import write_json
from thenewboston.utils.format import format_address
from thenewboston.utils.network import post
from thenewboston.utils.signed_requests import generate_signed_request

"""
Create invalid block
POST /invalid_blocks
"""


def run(send_to_bank=False):
    """
    Generate invalid block
    """

    # Signed request
    sk = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'cv_nid'))
    signed_request = generate_signed_request(
        data={
            'block': {
                'account_number': '0cdd4ba04456ca169baca3d66eace869520c62fe84421329086e03d91a68acdb',
                'message': {
                    'balance_key': 'ce51f0d9facaa7d3e69657429dd3f961ce70077a8efb53dcda508c7c0a19d2e3',
                    'txs': [
                        {
                            'amount': 12.5,
                            'recipient': '484b3176c63d5f37d808404af1a12c4b9649cd6f6769f35bdf5a816133623fbc'
                        },
                        {
                            'amount': 1,
                            'recipient': '5e12967707909e62b2bb2036c209085a784fabbc3deccefee70052b6181c8ed8'
                        },
                        {
                            'amount': 4,
                            'recipient': 'ad1f8845c6a1abb6011a2a434a079a087c460657aad54329a84b406dce8bf314'
                        }
                    ]
                },
                'signature': 'ee5a2f2a2f5261c1b633e08dd61182fd0db5604c853ebd8498f6f28ce8e2ccbbc38093918610ea88a7ad47c7f3192ed955d9d1529e7e390013e43f25a5915c0f'
            },
            'block_identifier': '65ae26192dfb9ec41f88c6d582b374a9b42ab58833e1612452d7a8f685dcd4d5',
            'primary_validator_node_identifier': PV_NID_ACCOUNT_NUMBER
        },
        nid_signing_key=sk
    )

    if send_to_bank:
        send_request_to_bank(signed_request)

    write_json(
        os.path.join(SIGNED_REQUESTS_DIR, 'invalid-block-request.json'),
        signed_request
    )


def send_request_to_bank(signed_request):
    """
    Send request to bank
    """

    node_address = format_address(
        ip_address='192.168.1.232',
        port=8000,
        protocol='http'
    )
    url = f'{node_address}/invalid_blocks'
    results = post(url=url, body=signed_request)

    if isinstance(results, dict):
        for k, v in results.items():
            print(f'{k}: {v}')

    print(results)

    write_json(
        os.path.join(SIGNED_REQUESTS_DIR, 'invalid-block-response.json'),
        results
    )


if __name__ == '__main__':
    run(send_to_bank=True)



import os

from playground.config import SIGNED_REQUESTS_DIR, SIGNING_KEY_DIR
from thenewboston.accounts.key_files import read_signing_key_file
from thenewboston.utils.files import write_json
from thenewboston.utils.format import format_address
from thenewboston.utils.network import post
from thenewboston.utils.signed_requests import generate_signed_request

"""
Creates a signed connection request that is used by the client to send to the target node
POST /connection_requests
"""


def run(send_to_node=False):
    """
    Generate signed connection request
    """

    # Signed request
    sk = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'bank_nid'))
    signed_request = generate_signed_request(
        data={
            'ip_address': '192.168.1.232',
            'port': '8000',
            'protocol': 'http'
        },
        nid_signing_key=sk
    )

    if send_to_node:
        send_request_to_node(signed_request)

    write_json(
        os.path.join(SIGNED_REQUESTS_DIR, 'connection-request.json'),
        signed_request
    )


def send_request_to_node(signed_request):
    """
    Send connection request to node
    """

    node_address = format_address(
        ip_address='192.168.1.75',
        port=8000,
        protocol='http'
    )
    url = f'{node_address}/connection_requests'
    results = post(url=url, body=signed_request)

    if isinstance(results, dict):
        for k, v in results.items():
            print(f'{k}: {v}')

    print(results)


if __name__ == '__main__':
    run(send_to_node=True)



from setuptools import setup, find_packages

setup(
    install_requires=[
        'Django==3.0.7',
        'PTable==0.9.2',
        'PyNaCl==1.3.0',
        'djangorestframework==3.11.0',
        'pycodestyle==2.6.0',
        'requests~=2.23.0',
    ],
    name='thenewboston',
    packages=find_packages(
        exclude=['tests', 'tests.*']
    ),
    version='0.0.4',
)



import os

from playground.config import (
    BANK_ACCOUNT_NUMBER,
    BANK_TX_FEE,
    BLOCKS_DIR,
    BUCKY_ACCOUNT_NUMBER,
    PV_ACCOUNT_NUMBER,
    PV_TX_FEE,
    SIGNING_KEY_DIR,
    TREASURY_ACCOUNT_NUMBER
)
from playground.utils import get_account_balance_lock
from thenewboston.accounts.key_files import read_signing_key_file
from thenewboston.blocks.block import generate_block
from thenewboston.blocks.signatures import generate_signature
from thenewboston.utils.files import write_json
from thenewboston.utils.format import format_address
from thenewboston.utils.network import post
from thenewboston.utils.tools import sort_and_encode
from thenewboston.verify_keys.verify_key import encode_verify_key, get_verify_key


def run(send_to_pv=False):
    """
    Create block used for:
    - POST /bank_blocks
    - Bank > PV
    """

    treasury_signing_key = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'treasury'))
    account_number = get_verify_key(signing_key=treasury_signing_key)

    balance_lock = get_account_balance_lock(account_number=TREASURY_ACCOUNT_NUMBER)
    transactions = [
        {
            'amount': BANK_TX_FEE,
            'recipient': BANK_ACCOUNT_NUMBER,
        },
        {
            'amount': PV_TX_FEE,
            'recipient': PV_ACCOUNT_NUMBER,
        },
        {
            'amount': 4.125,
            'recipient': BUCKY_ACCOUNT_NUMBER,
        }
    ]
    block = generate_block(
        account_number=account_number,
        balance_lock=balance_lock,
        signing_key=treasury_signing_key,
        transactions=transactions
    )

    bank_nid_sk = read_signing_key_file(os.path.join(SIGNING_KEY_DIR, 'bank_nid'))
    bank_nid = get_verify_key(signing_key=bank_nid_sk)
    bank_nid = encode_verify_key(verify_key=bank_nid)
    message = sort_and_encode(block)

    signed_block = {
        'block': block,
        'node_identifier': bank_nid,
        'signature': generate_signature(message=message, signing_key=bank_nid_sk)
    }

    write_json(
        os.path.join(BLOCKS_DIR, 'bank-blocks-request.json'),
        signed_block
    )

    if send_to_pv:
        send_request_to_pv(signed_block)


def send_request_to_pv(signed_request):
    """
    Send request to PV
    """

    node_address = format_address(
        ip_address='192.168.1.75',
        port=8000,
        protocol='http'
    )
    url = f'{node_address}/bank_blocks'
    results = post(url=url, body=signed_request)

    if isinstance(results, dict):
        for k, v in results.items():
            print(f'{k}: {v}')

    print(results)

    write_json(
        os.path.join(BLOCKS_DIR, 'bank-blocks-response.json'),
        results
    )


if __name__ == '__main__':
    run(send_to_pv=True)



from playground.config import TREASURY_ACCOUNT_NUMBER
from thenewboston.utils.format import format_address
from thenewboston.utils.network import fetch


def get_account_balance_lock(*, account_number):
    """
    Return the balance lock for the given account
    """

    bank_address = format_address(
        ip_address='192.168.1.75',
        port=8000,
        protocol='http'
    )
    url = f'{bank_address}/account_balance_lock/{account_number}'
    results = fetch(url=url, headers={})
    return results['balance_lock']


if __name__ == '__main__':
    get_account_balance_lock(account_number=TREASURY_ACCOUNT_NUMBER)



def format_address(*, ip_address, port, protocol):
    """
    Format address
    """

    port = f':{port}' if port else ''
    return f'{protocol}://{ip_address}{port}'


def format_node_address(*, node):
    """
    Format address for node
    """

    ip_address = node.ip_address
    port = f':{node.port}' if node.port else ''
    protocol = node.protocol
    return f'{protocol}://{ip_address}{port}'
